import { Http, IDictionary, IRequestConfig } from '@zeedhi/core';

interface RouteStructure {
  resource: string;
  uniqueKey: string;
  operation: 'create' | 'read' | 'update' | 'delete';
}

/**
 * Classe para realizar a conversão de requisições para o padrão REST
 */
export class HttpCRUDTranslator {
  // Mapeamento entre rotas do back - rotas do mirage
  private static routeMap: IDictionary<RouteStructure> = {
    '/inventarios/save': {
      resource: 'inventarios', uniqueKey: 'NRINVENTRETI', operation: 'create',
    },
    '/inventarios/delete': {
      resource: 'inventarios', uniqueKey: 'NRINVENTRETI', operation: 'delete',
    },
    '/servicosCardapioEfetivos/update': {
      resource: 'servicosCardapioEfetivos', uniqueKey: 'id', operation: 'update',
    },
    '/clientesDoServicoEfetivos/update': {
      resource: 'clientesDoServicoEfetivos', uniqueKey: 'id', operation: 'update',
    },
    '/produtosInventario/delete': {
      resource: 'produtosInventario', uniqueKey: 'CDPRODUTO', operation: 'delete',
    },
    '/produtosInventario/save': {
      resource: 'produtosInventario', uniqueKey: 'CDPRODUTO', operation: 'create',
    },
    '/produtosInventario/update': {
      resource: 'produtosInventario', uniqueKey: 'CDPRODUTO', operation: 'update',
    },
  };

  public static post(url: string, dataRequest?: any, config?: IRequestConfig): Promise<any> {
    // se estiver online não precisa realizar a conversão
    if (navigator.onLine) return Http.post(url, dataRequest, config);

    const structure = this.routeMap[url];
    const { resource, uniqueKey, operation } = structure;

    // Trata operação de read, convertendo o filtro para query string
    if (operation === 'read') {
      const query = dataRequest ? dataRequest.reduce(
        (result: string, filter: any) => (`${result}${filter.name}=${filter.value}&`), '?',
      ).slice(0, -1) : '';
      return Http.get(`${resource}${query}`, config);
    }

    let data = dataRequest;
    if (data?.requestType === 'DataSet') data = data.dataset;

    // Trata operação de create, realizando 1 post para cada linha criada
    if (operation === 'create') {
      if (!Array.isArray(data)) return Http.post(`${resource}`, data);
      if (Array.isArray(data) && data?.length === 1) return Http.post(`${resource}`, data[0]);

      const promises = data.map(async (row) => Http.post(resource, row));
      return Promise.all(promises);
    }

    // Trata operação de update, realizando 1 put para cada linha editada
    if (operation === 'update') {
      if (!Array.isArray(data)) return Http.put(`${resource}/${data[uniqueKey]}`, data);
      if (Array.isArray(data) && data?.length === 1) return Http.post(`${resource}`, data[0]);

      const promises = data.map(async (row) => Http.put(`${resource}/${row[uniqueKey]}`, row));
      return Promise.all(promises);
    }

    // Trata operação de delete, passando a chave única da linha na url
    if (operation === 'delete') {
      if (Array.isArray(data)) return Http.delete(`${resource}/${data[0][uniqueKey]}`, config);
      return Http.delete(`${resource}/${data[uniqueKey]}`, config);
    }

    // Se nenhuma operação for encontrada, apenas retorna null
    return new Promise(() => null);
  }
}
